<?php
/*
Plugin Name: Additonal Fees for WooCommerce Cart and Checkout
Description: A plugin to manage additional fees on the WooCommerce cart and checkout pages.
Version: 1.0
Author: Primary Digital Marketing (PrimaryDM)
*/

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

// Hook for adding admin menus
add_action('admin_menu', 'cwcf_custom_fees_menu');

// Action function for the above hook
function cwcf_custom_fees_menu() {
    add_submenu_page(
        'woocommerce',
        __('Custom Fees', 'custom-woocommerce-fees'),
        __('Custom Fees', 'custom-woocommerce-fees'),
        'manage_woocommerce',
        'custom-woocommerce-fees',
        'cwcf_custom_fees_page'
    );
}

// The admin page
function cwcf_custom_fees_page() {
    // Check user capabilities
    if (!current_user_can('manage_woocommerce')) {
        return;
    }

    // Check if there's a delete action
    if (isset($_GET['action']) && $_GET['action'] == 'delete' && isset($_GET['fee_index'])) {
        check_admin_referer('cwcf_delete_fee_action');

        $fee_index = intval($_GET['fee_index']);
        $fee_data = get_option('cwcf_fee_data', []);
        if (isset($fee_data[$fee_index])) {
            unset($fee_data[$fee_index]);
            update_option('cwcf_fee_data', array_values($fee_data)); // Reindex array
            echo '<div id="message" class="updated fade"><p>' . __('Fee deleted.', 'custom-woocommerce-fees') . '</p></div>';

            // Redirect to the same page without the delete action in URL
            wp_redirect(admin_url('admin.php?page=custom-woocommerce-fees'));
            exit;
        }
    }

    // Save posted data, if any
    if (isset($_POST['cwcf_fee_data'])) {
        check_admin_referer('cwcf_save_fee_data_action', 'cwcf_save_fee_data_nonce');

        // Save the data here
        $fee_data = [];
        foreach ($_POST['cwcf_fee_data'] as $fee) {
            if (empty($fee['label']) || empty($fee['amount'])) {
                continue; // Skip empty rows
            }
            // Validation and sanitization needed here
            $sanitized_fee = array_map('sanitize_text_field', $fee);
            // Ensure categories are stored as an array
            $sanitized_fee['categories'] = isset($fee['categories']) ? array_map('intval', $fee['categories']) : [];
            $fee_data[] = $sanitized_fee;
        }

        if (update_option('cwcf_fee_data', $fee_data)) {
            echo '<div id="message" class="updated fade"><p>' . __('Fees updated.', 'custom-woocommerce-fees') . '</p></div>';
        } else {
            error_log('Failed to update option cwcf_fee_data: ' . print_r($fee_data, true));
            echo '<div id="message" class="error fade"><p>' . __('Failed to update fees. Please check the error log for details.', 'custom-woocommerce-fees') . '</p></div>';
        }
    }

    // Fetch saved data, if any
    $fee_data = get_option('cwcf_fee_data', []);

    // Fetch WooCommerce categories for the dropdown
    $product_categories = get_terms('product_cat', array('hide_empty' => false));
    ?>
    <div class="wrap">
        <h1><?php _e('Custom WooCommerce Fees', 'custom-woocommerce-fees'); ?></h1>
        <form method="post" action="">
            <?php wp_nonce_field('cwcf_save_fee_data_action', 'cwcf_save_fee_data_nonce'); ?>

            <table class="wp-list-table widefat fixed striped">
                <thead>
                    <tr>
                        <th><?php _e('Fee Label', 'custom-woocommerce-fees'); ?></th>
                        <th><?php _e('Fee Amount', 'custom-woocommerce-fees'); ?></th>
                        <th><?php _e('Products', 'custom-woocommerce-fees'); ?></th>
                        <th><?php _e('Calculate Fee', 'custom-woocommerce-fees'); ?></th>
                        <th><?php _e('Categories (if applicable)', 'custom-woocommerce-fees'); ?></th>
                        <th><?php _e('Actions', 'custom-woocommerce-fees'); ?></th>
                    </tr>
                </thead>
                <tbody>
                    <?php foreach ($fee_data as $index => $fee): ?>
                    <tr>
                        <td><input type="text" name="cwcf_fee_data[<?php echo $index; ?>][label]" value="<?php echo esc_attr($fee['label']); ?>" /></td>
                        <td><input type="text" name="cwcf_fee_data[<?php echo $index; ?>][amount]" value="<?php echo esc_attr($fee['amount']); ?>" /></td>
                        <td>
                            <select name="cwcf_fee_data[<?php echo $index; ?>][products]" class="cwcf-products-dropdown">
                                <option value="all" <?php selected($fee['products'], 'all'); ?>><?php _e('All Products', 'custom-woocommerce-fees'); ?></option>
                                <option value="category" <?php selected($fee['products'], 'category'); ?>><?php _e('Products in Category', 'custom-woocommerce-fees'); ?></option>
                            </select>
                        </td>
                        <td>
                            <select name="cwcf_fee_data[<?php echo $index; ?>][calculate_fee]">
                                <option value="multiply_by_quantity" <?php selected($fee['calculate_fee'], 'multiply_by_quantity'); ?>><?php _e('Multiply by Quantity', 'custom-woocommerce-fees'); ?></option>
                                <option value="one_fee_per_cart" <?php selected($fee['calculate_fee'], 'one_fee_per_cart'); ?>><?php _e('One Fee Per Cart', 'custom-woocommerce-fees'); ?></option>
                            </select>
                        </td>
                        <td>
                            <select name="cwcf_fee_data[<?php echo $index; ?>][categories][]" class="cwcf-categories-field" multiple style="<?php echo ($fee['products'] !== 'category' ? 'display:none;' : ''); ?>">
                                <?php foreach ($product_categories as $category): ?>
                                    <option value="<?php echo esc_attr($category->term_id); ?>" <?php echo in_array($category->term_id, $fee['categories']) ? 'selected' : ''; ?>>
                                        <?php echo esc_html($category->name); ?>
                                    </option>
                                <?php endforeach; ?>
                            </select>
                        </td>
                        <td>
                            <a href="<?php echo esc_url(wp_nonce_url(admin_url('admin.php?page=custom-woocommerce-fees&action=delete&fee_index=' . $index), 'cwcf_delete_fee_action')); ?>" class="delete-fee"><?php _e('Delete', 'custom-woocommerce-fees'); ?></a>
                        </td>
                    </tr>
                    <?php endforeach; ?>
                    <tr id="new-fee-row" style="display: none;">
                        <td><input type="text" name="cwcf_fee_data[new][label]" /></td>
                        <td><input type="text" name="cwcf_fee_data[new][amount]" /></td>
                        <td>
                            <select name="cwcf_fee_data[new][products]" class="cwcf-products-dropdown">
                                <option value="all"><?php _e('All Products', 'custom-woocommerce-fees'); ?></option>
                                <option value="category"><?php _e('Products in Category', 'custom-woocommerce-fees'); ?></option>
                            </select>
                        </td>
                        <td>
                            <select name="cwcf_fee_data[new][calculate_fee]">
                                <option value="multiply_by_quantity"><?php _e('Multiply by Quantity', 'custom-woocommerce-fees'); ?></option>
                                <option value="one_fee_per_cart"><?php _e('One Fee Per Cart', 'custom-woocommerce-fees'); ?></option>
                            </select>
                        </td>
                        <td><!-- Actions for new row --></td>
                    </tr>
                </tbody>
            </table>
            <p><button type="button" class="button" id="toggle_new_fee_row"><?php _e('Add Fee', 'custom-woocommerce-fees'); ?></button></p>
            <?php submit_button(__('Save Changes', 'custom-woocommerce-fees'), 'primary', 'save_changes'); ?>
        </form>
    </div>
    <?php // Close PHP tag before the script ?>
<script type="text/javascript">
jQuery(document).ready(function($) {
    // Function to toggle category field based on product selection
    function toggleCategoryField($select) {
        var $categoriesField = $select.closest('tr').find('.cwcf-categories-field');
        if ($select.val() === 'category') {
            $categoriesField.show();
        } else {
            $categoriesField.hide();
        }
    }

    // Handle product type change
    $(document).on('change', '.cwcf-products-dropdown', function() {
        toggleCategoryField($(this));
    });

    // Handle adding new fee row
    $('#toggle_new_fee_row').on('click', function() {
        $('#new-fee-row').toggle();
    });
});
</script>

<?php
}

// Apply custom fees as line items to the Cart and checkout pages
add_action('woocommerce_cart_calculate_fees', 'apply_custom_fees', 10, 1);

function apply_custom_fees($cart) {
    if (is_admin() && !defined('DOING_AJAX')) {
        return;
    }

    // Fetch saved fee data
    $fee_data = get_option('cwcf_fee_data', []);

    foreach ($fee_data as $fee) {
        $apply_fee = false;

        if ($fee['products'] === 'all') {
            $apply_fee = true;
        } elseif ($fee['products'] === 'category' && !empty($fee['categories'])) {
            $categories = $fee['categories'];

            // Check if the cart contains products from the specified categories
            foreach (WC()->cart->get_cart() as $cart_item) {
                $product_cats = wp_get_post_terms($cart_item['product_id'], 'product_cat', array('fields' => 'ids'));
                if (array_intersect($categories, $product_cats)) {
                    $apply_fee = true;
                    break;
                }
            }
        }

        if ($apply_fee) {
            if ($fee['calculate_fee'] === 'multiply_by_quantity') {
                // Calculate fee by multiplying amount by the quantity
                $amount = floatval($fee['amount']) * WC()->cart->cart_contents_count;
            } else {
                // Default to one fee per cart
                $amount = floatval($fee['amount']);
            }

            if ($amount > 0) {
                $cart->add_fee($fee['label'], $amount);
            }
        }
    }
}
?>